﻿using AutoMapper;
using NPOI.HSSF.UserModel;
using NPOI.SS.UserModel;
using NPOI.SS.Util;
using NPOI.XSSF.UserModel;
using Performance.DtoModels;
using Performance.EntityModels;
using Performance.Infrastructure;
using Performance.Repository;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;

namespace Performance.Services
{
    public class PerExcelService : IAutoInjection
    {
        private PerSheetService _perSheetService;
        private PerHeaderService _perHeaderService;
        private PerforImSheetRepository _perforImSheetRepository;
        private PerforImDataRepository _perforImDataRepository;
        private PerforImHeaderRepository _perforImHeaderRepository;
        public PerExcelService(PerSheetService perSheetService,
            PerHeaderService perHeaderService,
            PerforImSheetRepository perforImSheetRepository,
            PerforImDataRepository perforImDataRepository,
            PerforImHeaderRepository perforImHeaderRepository)
        {
            _perSheetService = perSheetService;
            _perHeaderService = perHeaderService;
            _perforImSheetRepository = perforImSheetRepository;
            _perforImDataRepository = perforImDataRepository;
            _perforImHeaderRepository = perforImHeaderRepository;
        }

        /// <summary>
        /// 导入excel数据
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public PerExcel Import(string path)
        {
            PerExcel excel = new PerExcel
            {
                Path = path,
                FileName = FileHelper.GetFileNameNoExtension(path),
                Version = FileHelper.GetExtension(path) == ".xlsx" ? ExcelVersion.xlsx : ExcelVersion.xls,
                PerSheet = new List<PerSheet>()
            };
            using (FileStream fs = new FileStream(path, FileMode.Open))
            {
                IWorkbook workbook = (excel.Version == ExcelVersion.xlsx)
                    ? (IWorkbook)(new XSSFWorkbook(fs))
                    : (IWorkbook)(new HSSFWorkbook(fs));
                for (int i = 0; i < workbook.NumberOfSheets; i++)
                {
                    var sheet = workbook.GetSheetAt(i);
                    if (SheetType.Unidentifiable != _perSheetService.GetSheetType(sheet.SheetName))
                    {
                        var st = _perSheetService.Sheet(sheet);
                        excel.PerSheet.Add(st);
                    }
                }
                return excel;
            }
        }


        /// <summary>
        /// 复制报表基础配置
        /// </summary>
        /// <param name="iD"></param>
        public void Copy(int iD)
        {
            //throw new NotImplementedException();
        }

        /// <summary>
        /// 计算绩效
        /// </summary>
        /// <param name="excel"></param>
        public List<PerSheet> ProcessCompute(PerExcel excel)
        {
            return _perSheetService.ProcessCompute(excel);
        }

        /// <summary>
        /// 保存绩效结果
        /// </summary>
        /// <param name="perSheets"></param>
        /// <param name="allotId"></param>
        /// <param name="source">数据来源 1 excel 导入 2 计算</param>
        public void Save(List<PerSheet> perSheets, int allotId, int source)
        {
            foreach (var sheet in perSheets)
            {
                var imsheet = new im_sheet { AllotID = allotId, SheetName = sheet.SheetName, Source = source, SheetType = (int)sheet.SheetType };
                _perforImSheetRepository.Add(imsheet);

                foreach (var header in sheet.PerHeader)
                {
                    var imheader = Mapper.Map<im_header>(header);
                    imheader.SheetID = imsheet.ID;
                    _perforImHeaderRepository.Add(imheader);
                    if (header.IsHasChildren)
                    {
                        foreach (var child in header.Children)
                        {
                            var imheaderChild = Mapper.Map<im_header>(child);
                            imheaderChild.SheetID = imsheet.ID;
                            imheaderChild.ParentID = imheader.ID;
                            _perforImHeaderRepository.Add(imheaderChild);
                        }
                    }
                }
                var dataList = sheet.PerData.Select(t => (PerData)t);
                foreach (var data in dataList)
                {
                    var imdata = Mapper.Map<im_data>(data);
                    imdata.SheetID = imsheet.ID;
                    _perforImDataRepository.Add(imdata);
                }
            }

            //throw new NotImplementedException();
        }

        public IWorkbook ExportCompute(List<PerSheet> sheetList)
        {
            IWorkbook workbook = new XSSFWorkbook();
            var cellstyle = workbook.CreateCellStyle();
            cellstyle.VerticalAlignment = VerticalAlignment.Center;
            cellstyle.Alignment = HorizontalAlignment.Center;

            foreach (var sheet in sheetList)
            {
                ISheet exportSheet = workbook.CreateSheet(sheet.SheetName);
                //创建列头行
                IRow row = null, childRow = null;
                foreach (var header in sheet.PerHeader.OrderBy(t => t.PointCell))
                {
                    row = CreateRow(cellstyle, exportSheet, row, header);
                    //创建二级列头
                    if (header.IsHasChildren)
                    {
                        foreach (var child in header.Children.OrderBy(t => t.PointCell))
                        {
                            childRow = CreateRow(cellstyle, exportSheet, childRow, child);
                        }
                    }
                }
                //反转列头
                var headList = _perHeaderService.GetPerHeaderReverse(sheet.PerHeader);
                var dataList = sheet.PerData.Select(t => (PerData)t);
                var maxrow = headList.Max(t => t.PointRow);
                //循环核算单元 创建数据行
                var accountingUnitList = dataList.OrderBy(t => t.RowNumber).Select(t => t.AccountingUnit).Distinct();
                for (int i = 0; i < accountingUnitList.Count(); i++)
                {
                    var accountingUnit = accountingUnitList.ElementAt(i);
                    IRow dataRow = exportSheet.CreateRow(i + maxrow + 2);
                    dataRow.CreateCell(0).SetCellValue(accountingUnit);
                    dataRow.GetCell(0).CellStyle = cellstyle;
                    foreach (var header in headList.OrderBy(t => t.PointCell))
                    {
                        var cellValue = dataList.FirstOrDefault(t => t.AccountingUnit == accountingUnit && t.TypeName == header.CellValue)?.CellValue;
                        if (cellValue.HasValue && cellValue.Value > 0)
                        {
                            dataRow.CreateCell(header.PointCell).SetCellValue(Convert.ToDouble(cellValue.Value));
                            dataRow.GetCell(header.PointCell).CellStyle = cellstyle;
                        }
                    }
                }
            }
            return workbook;
        }

        /// <summary>
        /// 创建行
        /// </summary>
        /// <param name="cellstyle"></param>
        /// <param name="exportSheet"></param>
        /// <param name="row"></param>
        /// <param name="header"></param>
        /// <returns></returns>
        private IRow CreateRow(ICellStyle cellstyle, ISheet exportSheet, IRow row, PerHeader header)
        {
            if (header.IsMerge)
            {
                var cellRange = new CellRangeAddress(header.PointRow, header.PointRow + header.MergeRow - 1, header.PointCell, header.PointCell + header.MergeCell - 1);
                exportSheet.AddMergedRegion(cellRange);
            }
            row = row ?? exportSheet.CreateRow(header.PointRow);
            row.CreateCell(header.PointCell).SetCellValue(header.CellValue);
            row.GetCell(header.PointCell).CellStyle = cellstyle;
            return row;
        }

        /// <summary>
        /// 计算最终数据
        /// </summary>
        /// <param name="excel"></param>
        public void Compute(PerExcel excel)
        {
            //throw new NotImplementedException();
        }
    }
}
